/****************************
 * IBM Confidential
 * Licensed Materials - Property of IBM
 *
 * IBM Rational Developer Traveler
 *
 * (C) Copyright IBM Corporation 2013, 2014.
 *
 * The source code for this program is not published or otherwise divested of its trade secrets,
 * irrespective of what has been deposited with the U.S. Copyright Office.
 */
/*******************************************************************************
 * @license
 * Copyright (c) 2010, 2012 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License v1.0
 * (http://www.eclipse.org/legal/epl-v10.html), and the Eclipse Distribution
 * License v1.0 (http://www.eclipse.org/org/documents/edl-v10.html).
 *
 * Contributors: IBM Corporation - initial API and implementation
 *               Alex Lakatos - fix for bug#369781
 ******************************************************************************/
define([], function() {

	function StyleParser(styler) {
		this._styler = styler;
		this.createLinks = true;
	}

	StyleParser.prototype = {

		setWhitespacesVisible: function(visible) {
			this._styler.setWhitespacesVisible(visible);
		},
		setCreateLinks: function(enabled) {
			this.createLinks = enabled;
		},

		_binarySearch: function (array, offset, inclusive, low, high) {
			var index;
			if (low === undefined) { low = -1; }
			if (high === undefined) { high = array.length; }
			while (high - low > 1) {
				index = Math.floor((high + low) / 2);
				if (offset <= array[index].start) {
					high = index;
				} else if (inclusive && offset < array[index].end) {
					high = index;
					break;
				} else {
					low = index;
				}
			}
			return high;
		},
		_getBaseModel: function(viewModel) {
			return viewModel.getBaseModel ? viewModel.getBaseModel() : viewModel;
		},
		parseCommentsAndTasks: function(viewModel) {
			var baseModel = this._getBaseModel(viewModel);
			return this._findCommentsAndTasks(baseModel, baseModel.getText());
		},
		_findCommentsAndTasks: function(baseModel, text, offset) {
			offset = offset || 0;
			var tasks = [];
			var scanner = this._styler.firstScanner, token;
			scanner.setText(text);
			var result = [];
			var singleLineComments = [], i;
			while ((token = scanner.nextToken())) {
				if (token === scanner.MULTILINE_COMMENT || token === scanner.MULTILINE_STRING) {
					result.push({
						start: scanner.getStartOffset() + offset,
						end: scanner.getOffset() + offset,
						type: token
					});
				}
				if (token === scanner.SINGLELINE_COMMENT || token === scanner.MULTILINE_COMMENT) {
					//TODO can we avoid this work if edition does not overlap comment?
					var commentTasks = this._computeTasks(baseModel, token, scanner.getStartOffset() + offset, scanner.getOffset() + offset);
					Array.prototype.push.apply(tasks, commentTasks);
					if(token === scanner.SINGLELINE_COMMENT) {
						singleLineComments.push( {
							start: scanner.getStartOffset() + offset,
							end: scanner.getOffset() + offset,
							type: token
						});
					}
				}
			}

			var comments = [];
			var currentLine;

			for(i = 0; i < singleLineComments.length; i++){
				var singleLineComment = singleLineComments[i];
				if(comments.length === 0){
					comments.push(singleLineComment);
					currentLine = baseModel.getLineAtOffset(singleLineComment.start);
				}else{
					var nextComment = singleLineComments[i];
					var nextLine = baseModel.getLineAtOffset(nextComment.start);
					if(nextLine === currentLine + 1){
						currentLine = nextLine;
						comments.push(nextComment);
					}else{
						if(comments.length > 1){
							// We found a multiline coment
							result.push({
								start: comments[0].start,
								end: comments[comments.length - 1].end,
								type: scanner.MULTILINE_COMMENT
							});
						}
						comments = [nextComment];
						currentLine = baseModel.getLineAtOffset(nextComment.start);
					}
				}
			}
			if(comments.length > 1){
				// We found a multiline coment
				result.push({
					start: comments[0].start,
					end: comments[comments.length - 1].end,
					type: scanner.MULTILINE_COMMENT
				});
			}

			return {comments: result, tasks: tasks};
		},

		_computeTasks: function(baseModel, type, commentStart, commentEnd) {
			var add = [];
			var scanner = this._styler.commentScanner;
			scanner.setText(baseModel.getText(commentStart, commentEnd));
			var token;
			while ((token = scanner.nextToken())) {
				var tokenStart = scanner.getStartOffset() + commentStart;
				if (token ===  scanner.TASK_TAG) {
					var end = baseModel.getLineEnd(baseModel.getLineAtOffset(tokenStart));
					if (type !== scanner.SINGLELINE_COMMENT) {
						end = Math.min(end, commentEnd);
					}
					add.push({start:tokenStart, end:end, text:baseModel.getText(tokenStart,end)});
				}
			}
			return add;
		},

		/** Returns an array of style objects for one line of the editor text. */
		getStyles: function(viewModel, comments, text, start) {
			var model = viewModel;
			if (model.getBaseModel) {
				start = model.mapOffset(start);
			}
			var end = start + text.length;

			var styles = [];

			// for any sub range that is not a comment, parse code generating tokens (keywords, numbers, brackets, line comments, etc)
			var offset = start;
			var scanner = this._styler.scanner;
			var startIndex = this._binarySearch(comments, start, true);
			for (var i = startIndex; i < comments.length; i++) {
				if (comments[i].start >= end) { break; }
				var commentStart = comments[i].start;
				var commentEnd = comments[i].end;
				if (offset < commentStart) {
					this._parse(viewModel, text.substring(offset - start, commentStart - start), offset, styles);
				}
				var type = comments[i].type, style;
				switch (type) {
					case scanner.MULTILINE_COMMENT: style = this._styler.getStyleForToken(scanner.MULTILINE_COMMENT); break;
					case scanner.MULTILINE_STRING: style = this._styler.getStyleForToken(scanner.MULTILINE_STRING); break;
				}
				var s = Math.max(offset, commentStart);
				var e = Math.min(end, commentEnd);
				if ((type === scanner.MULTILINE_COMMENT) && (this._styler.whitespacesVisible)) {
					this._parseComment(text.substring(s - start, e - start), s, styles, style, type);
				} else if (type ===  scanner.MULTILINE_STRING && (this._styler.whitespacesVisible || this._styler.stringScanner)) {
					this._parseString(text.substring(s - start, e - start), s, styles, this._styler.getStyleForToken(scanner.STRING), this._styler.stringScanner);
				} else {
					styles.push({start: s, end: e, style: style});
				}
				offset = commentEnd;
			}
			if (offset < end) {
				this._parse(viewModel, text.substring(offset - start, end - start), offset, styles);
			}
			if (model.getBaseModel) {
				for (var j = 0; j < styles.length; j++) {
					var length = styles[j].end - styles[j].start;
					styles[j].start = model.mapOffset(styles[j].start, true);
					styles[j].end = styles[j].start + length;
				}
			}
			return styles;
		},

		_parse: function(viewModel, text, offset, styles) {
			var scanner = this._styler.scanner;
			scanner.setText(text);
			var token;
			while ((token = scanner.nextToken())) {
				var tokenStart = scanner.getStartOffset() + offset;
				var style = null;
				switch (token) {
					case scanner.KEYWORD:
					case scanner.PARAMETER:
						style = this._styler.getStyleForToken(token);
						if(this.createLinks){
							linkStyles = this._styler.createLink(this._getBaseModel(viewModel), scanner.getData(), text, tokenStart, style);
							if(linkStyles) {
								styles.push.apply(styles, linkStyles);
								style = null;
							}
						}
						break;
					case scanner.MULTILINE_STRING:
					case scanner.STRING:
						if (this._styler.whitespacesVisible) {
							this._parseString(scanner.getData(), tokenStart, styles, this._styler.getStyleForToken(scanner.STRING));
							continue;
						} else {
							style = this._styler.getStyleForToken(scanner.STRING);
						}
						break;
					case scanner.SINGLELINE_COMMENT:
						this._parseComment(scanner.getData(), tokenStart, styles,  this._styler.getStyleForToken(scanner.SINGLELINE_COMMENT), token);
						continue;
					case scanner.MULTILINE_COMMENT:
						this._parseComment(scanner.getData(), tokenStart, styles, this._styler.getStyleForToken(scanner.MULTILINE_COMMENT), token);
						continue;
					case scanner.WHITE_TAB:
						if (this._styler.whitespacesVisible) {
							style = this._styler.getStyleForToken(scanner.WHITE_TAB);
						}
						break;
					case scanner.WHITE_SPACE:
						if (this._styler.whitespacesVisible) {
							style = this._styler.getStyleForToken(scanner.WHITE_SPACE);
						}
						break;
					default:
						style = this._styler.getStyleForToken(token);
						break;
				}
				if(style){
					styles.push({start: tokenStart, end: scanner.getOffset() + offset, style: style});
				}
			}
		},

		_parseComment: function(text, offset, styles, s, type) {
			var scanner = this._styler.commentScanner;
			scanner.setText(text);
			scanner.setType(type);
			var token;
			while ((token = scanner.nextToken())) {
				var tokenStart = scanner.getStartOffset() + offset;
				var style = s;
				switch (token) {
					case scanner.WHITE_TAB:
						if (this._styler.whitespacesVisible) {
							style = this._styler.getStyleForToken(scanner.WHITE_TAB);
						}
						break;
					case scanner.WHITE_SPACE:
						if (this._styler.whitespacesVisible) {
							style = this._styler.getStyleForToken(scanner.WHITE_SPACE);
						}
						break;
					case scanner.TASK_TAG:
						style = this._styler.getStyleForToken(scanner.TASK_TAG);
						break;
					default:
						break;
				}
				if (style) {
					styles.push({start: tokenStart, end: scanner.getOffset() + offset, style: style});
				}
			}
		},

		_parseString: function(text, offset, styles, s, scanner) {
			scanner = scanner || this._styler.whitespaceScanner;
			scanner.setText(text);
			var token;
			while ((token = scanner.nextToken())) {
				var tokenStart = scanner.getStartOffset() + offset;
				var style = s;
				switch (token) {
					case scanner.WHITE_TAB:
						if (this._styler.whitespacesVisible) {
							style = this._styler.getStyleForToken(scanner.WHITE_TAB);
						}
						break;
					case scanner.WHITE_SPACE:
						if (this._styler.whitespacesVisible) {
							style = this._styler.getStyleForToken(scanner.WHITE_SPACE);
						}
						break;
					case scanner.IDENTIFIER:
						style = this._styler.getStyleForToken(scanner.IDENTIFIER);
						break;
				}
				if (style) {
					styles.push({start: tokenStart, end: scanner.getOffset() + offset, style: style});
				}
			}
		},

		findMatchingBracket: function(viewModel, comments, offset) {
			var model = this._getBaseModel(viewModel);
			var brackets = this._styler.brackets;
			var bracket = model.getText(offset, offset + 1);
			var bracketIndex = brackets.indexOf(bracket, 0);
			if (bracketIndex === -1) { return -1; }
			var closingBracket;
			if (bracketIndex & 1) {
				closingBracket = brackets.substring(bracketIndex - 1, bracketIndex);
			} else {
				closingBracket = brackets.substring(bracketIndex + 1, bracketIndex + 2);
			}
			var lineIndex = model.getLineAtOffset(offset);
			var lineText = model.getLine(lineIndex);
			var lineStart = model.getLineStart(lineIndex);
			var lineEnd = model.getLineEnd(lineIndex);
			brackets = this._findBrackets(comments, bracket, closingBracket, lineText, lineStart, lineStart, lineEnd);
			for (var i=0; i<brackets.length; i++) {
				var sign = brackets[i] >= 0 ? 1 : -1;
				if (brackets[i] * sign - 1 === offset) {
					var level = 1;
					if (bracketIndex & 1) {
						i--;
						for (; i>=0; i--) {
							sign = brackets[i] >= 0 ? 1 : -1;
							level += sign;
							if (level === 0) {
								return brackets[i] * sign - 1;
							}
						}
						lineIndex -= 1;
						while (lineIndex >= 0) {
							lineText = model.getLine(lineIndex);
							lineStart = model.getLineStart(lineIndex);
							lineEnd = model.getLineEnd(lineIndex);
							brackets = this._findBrackets(comments, bracket, closingBracket, lineText, lineStart, lineStart, lineEnd);
							for (var j=brackets.length - 1; j>=0; j--) {
								sign = brackets[j] >= 0 ? 1 : -1;
								level += sign;
								if (level === 0) {
									return brackets[j] * sign - 1;
								}
							}
							lineIndex--;
						}
					} else {
						i++;
						for (; i<brackets.length; i++) {
							sign = brackets[i] >= 0 ? 1 : -1;
							level += sign;
							if (level === 0) {
								return brackets[i] * sign - 1;
							}
						}
						lineIndex += 1;
						var lineCount = model.getLineCount ();
						while (lineIndex < lineCount) {
							lineText = model.getLine(lineIndex);
							lineStart = model.getLineStart(lineIndex);
							lineEnd = model.getLineEnd(lineIndex);
							brackets = this._findBrackets(comments, bracket, closingBracket, lineText, lineStart, lineStart, lineEnd);
							for (var k=0; k<brackets.length; k++) {
								sign = brackets[k] >= 0 ? 1 : -1;
								level += sign;
								if (level === 0) {
									return brackets[k] * sign - 1;
								}
							}
							lineIndex++;
						}
					}
					break;
				}
			}
			return -1;
		},
		_findBrackets: function(comments, bracket, closingBracket, text, textOffset, start, end) {
			var result = [];
			var bracketToken = bracket.charCodeAt(0);
			var closingBracketToken = closingBracket.charCodeAt(0);
			// for any sub range that is not a comment, parse code generating tokens (keywords, numbers, brackets, line comments, etc)
			var offset = start, token;
			var scanner = this._styler.bracketScanner || this._styler.scanner; // bracketScanner is optional
			var startIndex = this._binarySearch(comments, start, true);
			for (var i = startIndex; i < comments.length; i++) {
				if (comments[i].start >= end) { break; }
				var commentStart = comments[i].start;
				var commentEnd = comments[i].end;
				if (offset < commentStart) {
					scanner.setText(text.substring(offset - start, commentStart - start));
					while ((token = scanner.nextToken())) {
						if (token === bracketToken) {
							result.push(scanner.getStartOffset() + offset - start + textOffset + 1);
						} else if (token === closingBracketToken) {
							result.push(-(scanner.getStartOffset() + offset - start + textOffset + 1));
						}
					}
				}
				offset = commentEnd;
			}
			if (offset < end) {
				scanner.setText(text.substring(offset - start, end - start));
				while ((token = scanner.nextToken())) {
					if (token === bracketToken) {
						result.push(scanner.getStartOffset() + offset - start + textOffset + 1);
					} else if (token === closingBracketToken) {
						result.push(-(scanner.getStartOffset() + offset - start + textOffset + 1));
					}
				}
			}
			return result;
		},

		repair: function(viewModel, comments, start, removedCharCount, addedCharCount) {
			var baseModel = this._getBaseModel(viewModel);
			var changeCount = addedCharCount - removedCharCount;
			var end = start + removedCharCount;
			var charCount = baseModel.getCharCount();
			var commentCount = comments.length;
			var lineStart = baseModel.getLineStart(baseModel.getLineAtOffset(start));
			var commentStart = this._binarySearch(comments, lineStart, true);
			var commentEnd = this._binarySearch(comments, end, false, commentStart - 1, commentCount);

			var ts;
			if (commentStart < commentCount && comments[commentStart].start <= lineStart && lineStart < comments[commentStart].end) {
				ts = comments[commentStart].start;
				if (ts > start) { ts += changeCount; }
			} else {
				if (commentStart === commentCount && commentCount > 0 && charCount - changeCount === comments[commentCount - 1].end) {
					ts = comments[commentCount - 1].start;
				} else {
					ts = lineStart;
				}
			}
			ts = baseModel.getLineStart(baseModel.getLineAtOffset(ts)); // the JCL scanner can only handle starting at the beginning of a line, cobol scanner doesn't care

			var te;
			if (commentEnd < commentCount) {
				te = comments[commentEnd].end;
				if (te > start) { te += changeCount; }
				commentEnd += 1;
			} else {
				commentEnd = commentCount;
				te = charCount;//TODO could it be smaller?
			}
			var text = baseModel.getText(ts, te), comment;
			var commentsAndTasks = this._findCommentsAndTasks(baseModel, text, ts), i;
			var newComments = commentsAndTasks.comments;

			for (i = commentStart; i < comments.length; i++) {
				comment = comments[i];
				if (comment.start > start) { comment.start += changeCount; }
				if (comment.start > start) { comment.end += changeCount; }
			}
			var redraw = (commentEnd - commentStart) !== newComments.length;
			if (!redraw) {
				for (i=0; i<newComments.length; i++) {
					comment = comments[commentStart + i];
					var newComment = newComments[i];
					if (comment.start !== newComment.start || comment.end !== newComment.end || comment.type !== newComment.type) {
						redraw = true;
						break;
					}
				}
			}
			var args = [commentStart, commentEnd - commentStart].concat(newComments);
			Array.prototype.splice.apply(comments, args);

			return {
				newComments: commentsAndTasks.comments,
				newTasks: commentsAndTasks.tasks,
				redraw: redraw,
				start: ts,
				end: te
			};
		}
	};

	return {StyleParser: StyleParser};

});