/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016, 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dojo/store/Memory", "dijit/tree/ObjectStoreModel", "dijit/Tree", "dojo/dom-attr",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/searchPage",
], function ( Memory, ObjectStoreModel, Tree, domAttr, searchPage ) {
	return {
		createTree : function ( resultJSONProject, streamId ) {
			resultJSONProject.sort(function(a,b) {
				if ( a.model == "project" && b.model == "project") {
					var _a = a.name;
					var _b = b.name;
					if (_a < _b)
						return -1;
					if (_a > _b)
						return 1;
					return 0;					
				} else if ( a.model == "instance" && b.model == "instance") {
					var _a = a.name;
					var _b = b.name;
					if (_a < _b)
						return -1;
					if (_a > _b)
						return 1;
					return 0;					
				} else if ( a.model == "type" && b.model == "type") {
					var _a = a.name;
					var _b = b.name;
					if (_a < _b)
						return -1;
					if (_a > _b)
						return 1;
					return 0;					
				} else {
					var _a = a.model;
					var _b = b.model;
					if (_a < _b)
						return -1;
					if (_a > _b)
						return 1;
					return 0;	
				}	
			});
			
			var myStore = new Memory( {
				data : resultJSONProject,
				getChildren : function ( object ) {
					return this.query( {
						parent : object.id
					} );
				}
			} );

			// Create the model
			var myModel = new ObjectStoreModel( {
				store : myStore,
				query : {
					root : true
				},
				mayHaveChildren : function ( item ) {
					// see if it is an instance
					return item.model != "instance";
				}
			} );

			var myTreeSearch = new Tree( {
				model : myModel,
				openOnClick : true,
				onDblClick : function ( item ) {
					if ( item.model == "instance" ) {
						openArtifactFile( streamId, item.compentId, item.fileItemId, item.project, item.project, item.type, item.metaType, item.name );
					}
				},
				getIconClass : function ( item, opened ) {
					if ( item.model == "instance" )
						return item.type + " display_icon";
					else {
						return opened
								? "dijitFolderOpened" : "dijitFolderClosed";
					}
				},
				// allow HTML markup in the tree
				_createTreeNode : function ( args ) {
					var tnode = new dijit._TreeNode( args );
					tnode.labelNode.innerHTML = args.label;
					return tnode;
				},
				getLabel : function ( item ) {
					var label = item.name;
					if ( item.model == "instance" ) {
						if ( item.metaType && item.metaType.length > 0 ) {
							label += " (" + item.metaType + ") ";
						}
						var context = "";
						label += " [" + item.label + "] ";
						label += " <button onclick='createReferencePane(\"" + item.name + "\",\"" + item.type + "\",\"" + item.project + "\",\"" + context
								+ "\",\"super\",\"" + item.fileItemId + "\",\"" + item.compentId + "\",\"" + streamId
								+ "\")' hidden='hidden' id='super" + item.name + "/" + item.parent + "'>" + searchPage[ "super" ] + "</button>";
						label += " <button onclick='createReferencePane(\"" + item.name + "\",\"" + item.type + "\",\"" + item.project + "\",\"" + context
								+ "\",\"sub\",\"" + item.fileItemId + "\",\"" + item.compentId + "\",\"" + streamId
								+ "\")' hidden='hidden' id='sub" + item.name + "/" + item.parent + "'>" + searchPage[ "sub" ] + "</button>";
					}
					return label;
				},
				_onNodeMouseEnter : function ( node, evt ) {
					if ( node.item.model == "instance" ) {
						domAttr.remove( "super" + node.item.name + "/" + node.item.parent, "hidden" );
						domAttr.remove( "sub" + node.item.name + "/" + node.item.parent, "hidden" );
					}
				},
				_onNodeMouseLeave : function ( node, evt ) {
					if ( node.item.model == "instance" ) {
						domAttr.set( "super" + node.item.name + "/" + node.item.parent, "hidden", "hidden" );
						domAttr.set( "sub" + node.item.name + "/" + node.item.parent, "hidden", "hidden" );

					}
				}
			} );
			return myTreeSearch;
		}
	};
} );