/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2017. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

define( [
		"dijit/layout/ContentPane", "dojo/string", "dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/PacbaseEditor",
		"dojo/i18n!com.ibm.teampdp.query.engine.web/ui/internal/page/nls/typeDict"
], function ( ContentPane, string, PacbaseEditor, typeDict ) {
	
		function sortElements(elements) {
			var components = elements.sort(function (a, b) {
				var _a = null;
				var _b = null;
				if (a.dataAggregate != undefined) {
					_a = a.dataAggregate.name;
					_b = b.dataAggregate.name;
				}
				else {
					if (a.radicalEntity != undefined) {
						var _a = a.radicalEntity.name;
						var _b = b.radicalEntity.name;
					}
				}
				if (_a != null && _b != null) {
					if (_a < _b)
						return -1;
					if (_a > _b)
						return 1;
					return 0;	
				}
			});	
			return components;
		}
		
		function getDetailLine (item, streamId, project) {
			
			// DataStructure case for segments list
			if ( item.dataDefinition != undefined &&  item.dataDefinition != "" ) {
				return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.dataDefinition[0], true, streamId, project); 
			}
			else {
				// Copybook case for selected segments list
				if ( item.dataAggregate != undefined &&  item.dataAggregate != "" ) {
					return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.dataAggregate[0], true, streamId, project); 
				}
				// errorLabel case for selected entities list
				if ( item.radicalEntity != undefined &&  item.radicalEntity != "" ) {
					return new com.ibm.teampdp.query.engine.web.ui.internal.page.jsModules.widgets.RadicalEntityWidget("", item.radicalEntity[0], true, streamId, project); 
				}
			}
		}
		
		return {			
		
		getDetailLines : function ( entity, type, project, streamId, compentId, fileItemId, headerTitle, descriptionTitle, headerGridTitle, hearderTooltips) {
			var detailLSLines = new ContentPane( {
				content : "<strong>" + headerTitle + "</strong><hr><P>"
						+ string.substitute( descriptionTitle, [
							typeDict[ type ]
						] ) + "</p>",
						style : {
							'overflow' : 'auto',
						},
			} );
			
			var components = null;
			// DataStructure case for segments list
			if (entity.components != undefined) {
				components = entity.components;
			} else {
				// Copybook case for selected segments list
				if (entity.entitiesSelected != undefined) {
					components = sortElements(entity.entitiesSelected);
				}
				// errorLabel case for selected segments list
				else if (entity.selectedEntities != undefined) {
					components = sortElements(entity.selectedEntities);
				}
			}
			
			if ( components != null && components.length != 0 ) {
				require( [
						'dojox/grid/DataGrid', 'dojo/data/ItemFileReadStore'
				], function ( DataGrid, ItemFileReadStore ) {
					/* set up data store */
					var store = new ItemFileReadStore( {
						data : {
							items : components
						}
					} );
					/* set up layout */
					var layout = [
						{
							name : headerGridTitle,
							field : "_item",
							width : "700px",
							formatter : function ( item ) {
								return getDetailLine(item, streamId, project);								
							},
						}
					];
					/*
					var hearderTooltips = null;
					if(entity["class"] == "PacErrorLabel") {
						hearderTooltips =  [PacbaseEditor.ERROR_LABEL_COLUMN_TOOL_TIP_SELECTED_LIST];
					}else if(entity["class"] == "PacCopybook") {
						hearderTooltips =  [PacbaseEditor.COPY_COLUMN_TOOL_TIP_SELECTED_LIST];
					}*/
					/* create a new grid */
					var lsLinesGrid = new DataGrid( {
						store : store,
						structure : layout,
						height : ( rppCurrentPaneGridHeight ) + "px",
						autoWidth : true, 
						selectable : true,
						canSort : function () {
							return false;
						},
						onHeaderCellMouseOver  : function (e) {
							dijit.showTooltip(hearderTooltips, e.cellNode);
						},
						onHeaderCellMouseOut : function (e) {
							dijit.hideTooltip(e.cellNode);
						}
					} );
					detailLSLines.addChild( lsLinesGrid );
				} );
			}
			return detailLSLines;
		}
	};
} );